let read_all in_c =
    let buf = ref "" in
    try
      while true do
        let line = input_line in_c in
        buf := !buf ^ line ^ "\n"
      done;
      !buf
    with End_of_file -> !buf

let file_exists f =
    try
        Unix.access f [Unix.F_OK]; true
    with Unix.Unix_error (_,_,_) -> false

let remove_if_exists f =
    if file_exists f then
        Sys.remove f
    else
        ()

let recursive_remove dirname =
    if Sys.command ("/bin/rm -rf " ^ dirname) <> 0 then
        Message.die ("Unable to recursively remove " ^ dirname)
    else ()

(* note: copy needs to be smart if second arg is a directory *)
let copy f1 f2 =
    if Sys.command ("/bin/cp " ^ f1 ^ " " ^ f2) <> 0 then
        Message.die ("Unable to copy: " ^ f1 ^ " --> " ^ f2)
    else ()

(* here, the second arg is always a dir *)
let copydir d1 d2 =
    if Sys.command ("/bin/cp -R " ^ d1 ^ " " ^ d2) <> 0 then
        Message.die ("Unable to copydir: " ^ d1 ^ " --> " ^ d2)
    else ()

let write_file str filename =
    let chan = open_out filename in
    output_string chan str;
    close_out chan

let write_file_opt stro filename =
    match stro with
        None -> ()
      | Some str -> write_file str filename

let mkdir d = Unix.mkdir d 0o755

type catable = String of string | File of string
let cat catables =
    let buf = ref "" in
    let cat_one = function
        String s -> buf := !buf ^ s
      | File f -> (let chan = open_in f in
                   buf := !buf ^ (read_all chan);
                   close_in chan)
    in
    List.iter cat_one catables;
    !buf

let cat_to_file catables filename =
    let s = cat catables in
    let outc = open_out filename in
    output_string outc s;
    close_out outc

let chop_in_two s n =
    let first = String.sub s 0 n in
    let second = String.sub s n (String.length s - n) in
    (first, second)

let textwrap s maxlen =
    (* start by stripping the newlines out of the string, replacing with
       spaces. *)
    let s = Str.global_replace (Str.regexp "\n") " " s in
    let buf = ref "" in
    let rec wrap s maxlen =
        if String.length s > maxlen then
        try
          let chop_pos =
            try
                let p = String.rindex_from s maxlen ' ' in
                if p = 0 then raise Not_found else p
                (* for some reason rindex_from keeps thinking there's a
                   space at 0 when there's not *)
            with
                Not_found -> (* rindex_from couldn't find a space, so fall into
                                wrap-at-next-space-forward mode *)
                    String.index_from s maxlen ' '
          in
          let (first, rest) = chop_in_two s chop_pos in
          (* drop the space character: *)
          let rest = String.sub rest 1 (String.length rest - 1) in
          buf := !buf ^ first ^ "\n";
          wrap rest maxlen
        with
            Not_found -> buf := !buf ^ s
            (* there are no spaces left in the whole string!  just give up.. *)
        else
          buf := !buf ^ s
    in
    wrap s maxlen;
    !buf
